@@ -113,7 +113,7 @@ class GrapheneInstigationTick(graphene.ObjectType):
         self._tick = check.inst_param(tick, "tick", InstigatorTick)
 
         super().__init__(
-            status=tick.status,
+            status=tick.status.value,
             timestamp=tick.timestamp,
             runIds=tick.run_ids,
             runKeys=tick.run_keys,
@@ -306,7 +306,7 @@ class GrapheneInstigationState(graphene.ObjectType):
             id=instigator_state.instigator_origin_id,
             selectorId=instigator_state.selector_id,
             name=instigator_state.name,
-            instigationType=instigator_state.instigator_type,
+            instigationType=instigator_state.instigator_type.value,
             status=(
                 GrapheneInstigationStatus.RUNNING
                 if instigator_state.is_running
