@@ -25,6 +25,7 @@ from ..asset_key import GrapheneAssetKey
 from ..dagster_types import GrapheneDagsterType, GrapheneDagsterTypeOrError, to_dagster_type
 from ..errors import GrapheneDagsterTypeNotFoundError, GraphenePythonError, GrapheneRunNotFoundError
 from ..execution import GrapheneExecutionPlan
+from ..inputs import GrapheneInputTag
 from ..logs.compute_logs import GrapheneCapturedLogs, GrapheneComputeLogs, from_captured_log_data
 from ..logs.events import (
     GrapheneDagsterRunEvent,
@@ -86,12 +87,27 @@ def parse_time_range_args(args):
     return before_timestamp, after_timestamp
 
 
-class GrapheneMaterializationCount(graphene.ObjectType):
-    partition = graphene.NonNull(graphene.String)
-    materializationCount = graphene.NonNull(graphene.Int)
+class GrapheneMaterializationCountSingleDimension(graphene.ObjectType):
+    materializationCounts = non_null_list(graphene.Int)
 
     class Meta:
-        name = "MaterializationCountByPartition"
+        name = "MaterializationCountSingleDimension"
+
+
+class GrapheneMaterializationCountGroupedByDimension(graphene.ObjectType):
+    materializationCountsGrouped = graphene.NonNull(graphene.List(non_null_list(graphene.Int)))
+
+    class Meta:
+        name = "MaterializationCountGroupedByDimension"
+
+
+class GraphenePartitionMaterializationCounts(graphene.Union):
+    class Meta:
+        types = (
+            GrapheneMaterializationCountSingleDimension,
+            GrapheneMaterializationCountGroupedByDimension,
+        )
+        name = "PartitionMaterializationCounts"
 
 
 class GrapheneAsset(graphene.ObjectType):
@@ -104,6 +120,7 @@ class GrapheneAsset(graphene.ObjectType):
         beforeTimestampMillis=graphene.String(),
         afterTimestampMillis=graphene.String(),
         limit=graphene.Int(),
+        tags=graphene.Argument(graphene.List(graphene.NonNull(GrapheneInputTag))),
     )
     assetObservations = graphene.Field(
         non_null_list(GrapheneObservationEvent),
@@ -142,6 +159,7 @@ class GrapheneAsset(graphene.ObjectType):
         partitionInLast = kwargs.get("partitionInLast")
         if partitionInLast and self._definition:
             partitions = self._definition.get_partition_keys()[-int(partitionInLast) :]
+        tags = kwargs.get("tags")
 
         events = get_asset_materializations(
             graphene_info,
@@ -149,6 +167,7 @@ class GrapheneAsset(graphene.ObjectType):
             partitions=partitions,
             before_timestamp=before_timestamp,
             after_timestamp=after_timestamp,
+            tags={tag["name"]: tag["value"] for tag in tags} if tags else None,
             limit=limit,
         )
         run_ids = [event.run_id for event in events]
@@ -289,7 +308,7 @@ class GrapheneRun(graphene.ObjectType):
         pipeline_run = record.pipeline_run
         super().__init__(
             runId=pipeline_run.run_id,
-            status=PipelineRunStatus(pipeline_run.status),
+            status=pipeline_run.status.value,
             mode=pipeline_run.mode,
         )
         self._pipeline_run = pipeline_run
@@ -343,7 +362,7 @@ class GrapheneRun(graphene.ObjectType):
             self.run_id, fileKey
         )
         log_data = graphene_info.context.instance.compute_log_manager.get_log_data(log_key)
-        return from_captured_log_data(graphene_info, log_data)
+        return from_captured_log_data(log_data)
 
     def resolve_executionPlan(self, graphene_info):
         if not (
@@ -782,6 +801,18 @@ class GrapheneJob(GraphenePipeline):
         interfaces = (GrapheneSolidContainer, GrapheneIPipelineSnapshot)
         name = "Job"
 
+    # doesn't inherit from base class
+    def __init__(self, external_pipeline, batch_loader=None):
+        super().__init__()
+        self._external_pipeline = check.inst_param(
+            external_pipeline, "external_pipeline", ExternalPipeline
+        )
+        # optional run loader, provided by a parent GrapheneRepository object that instantiates
+        # multiple pipelines
+        self._batch_loader = check.opt_inst_param(
+            batch_loader, "batch_loader", RepositoryScopedBatchLoader
+        )
+
 
 class GrapheneGraph(graphene.ObjectType):
     class Meta:
