@@ -22,6 +22,7 @@ from ...implementation.fetch_assets import (
     get_asset_node_definition_collisions,
     get_asset_nodes,
     get_assets,
+    unique_repos,
 )
 from ...implementation.fetch_backfills import get_backfill, get_backfills
 from ...implementation.fetch_instigators import (
@@ -50,7 +51,11 @@ from ...implementation.fetch_schedules import (
 )
 from ...implementation.fetch_sensors import get_sensor_or_error, get_sensors_or_error
 from ...implementation.fetch_solids import get_graph_or_error
-from ...implementation.loader import BatchMaterializationLoader, CrossRepoAssetDependedByLoader
+from ...implementation.loader import (
+    BatchMaterializationLoader,
+    CrossRepoAssetDependedByLoader,
+    ProjectedLogicalVersionLoader,
+)
 from ...implementation.run_config_schema import resolve_run_config_schema_or_error
 from ...implementation.utils import graph_selector_from_graphql, pipeline_selector_from_graphql
 from ..asset_graph import (
@@ -541,6 +546,7 @@ class GrapheneDagitQuery(graphene.ObjectType):
             AssetKey.from_graphql_input(asset_key) for asset_key in kwargs.get("assetKeys", [])
         )
 
+        repo = None
         if "group" in kwargs:
             group_name = kwargs.get("group").get("groupName")
             repo_sel = RepositorySelector.from_graphql_input(kwargs.get("group"))
@@ -584,6 +590,18 @@ class GrapheneDagitQuery(graphene.ObjectType):
         )
 
         depended_by_loader = CrossRepoAssetDependedByLoader(context=graphene_info.context)
+
+        if repo is not None:
+            repos = [repo]
+        else:
+            repos = unique_repos(result.external_repository for result in results)
+
+        projected_logical_version_loader = ProjectedLogicalVersionLoader(
+            instance=graphene_info.context.instance,
+            key_to_node_map={node.assetKey: node.external_asset_node for node in results},
+            repositories=repos,
+        )
+
         return [
             GrapheneAssetNode(
                 node.repository_location,
@@ -591,6 +609,7 @@ class GrapheneDagitQuery(graphene.ObjectType):
                 node.external_asset_node,
                 materialization_loader=materialization_loader,
                 depended_by_loader=depended_by_loader,
+                projected_logical_version_loader=projected_logical_version_loader,
             )
             for node in results
         ]
