@@ -3,9 +3,9 @@ from dagster_graphql.implementation.loader import RepositoryScopedBatchLoader
 from dagster_graphql.implementation.utils import capture_error, check_permission
 
 import dagster._check as check
-from dagster.core.host_representation import ExternalSensor, ExternalTargetData, SensorSelector
-from dagster.core.scheduler.instigation import InstigatorState
-from dagster.core.workspace.permissions import Permissions
+from dagster._core.host_representation import ExternalSensor, ExternalTargetData, SensorSelector
+from dagster._core.scheduler.instigation import InstigatorState
+from dagster._core.workspace.permissions import Permissions
 
 from ..implementation.fetch_sensors import (
     get_sensor_next_tick,
